@@ -1,9 +1,7 @@
-#!/usr/bin/env python
+#!/usr/bin/python
+# -*- coding: utf-8 -*-
 
-"""KhtEditor a source code editor by Khertan : Editor Window"""
-
-import sys
-import re
+"""KhtEditor a source code editor by Benoît HERVIER (Khertan) : Editor Window"""
 
 from PyQt4.QtGui import QMainWindow, \
     QHBoxLayout, \
@@ -17,77 +15,33 @@ from PyQt4.QtGui import QMainWindow, \
     QFrame, \
     QLabel, \
     QPushButton, \
-    QFont, \
     QKeySequence, \
     QLayout, \
     QMenu, \
     QAction, \
     QApplication, \
     QIcon, \
-    QMessageBox
+    QMessageBox, \
+    QToolButton
 
 from PyQt4.QtCore import QFileInfo, \
     Qt, \
-    QTimer, \
     QSettings, \
     pyqtSlot,pyqtSignal
-    
-from PyQt4.QtCore import Qt
 
-from plugins.plugins_api import init_plugin_system, filter_plugins_by_capability, find_plugins, Plugin
+from plugins.plugins_api import init_plugin_system, filter_plugins_by_capability, find_plugins
 import editor
 #import editor_frame
-from subprocess import *
+from subprocess import Popen
 import commands
 import os
 
-LANGUAGES = (('.R','R'),
-            ('.ada','ada'),
-            ('.c','c'),
-            ('.changelog','changelog'),
-            ('.cpp','cpp'),
-            ('.csharp','csharp'),
-            ('.desktop','desktop'),
-            ('.css','css'),
-            ('.diff','diff'),
-            ('.fort','fortran'),
-            ('.gtkrc','gtkrc'),
-            ('.haskell','haskell'),
-            ('.html','html'),
-            ('.idl','idl'),
-            ('.ini','ini'),
-            ('.java','java'),
-            ('.js','javascript'),
-            ('.tex','latex'),
-            ('.lua','lua'),
-            ('makefile','makefile'),
-            ('markdown','markdown'),
-            ('.msil','msil'),
-            ('nemerle','nemerle'),
-            ('octave','octave'),
-            ('.pas','pascal'),
-            ('.pl','perl'),
-            ('.php','php'),
-            ('.po','po'),
-            ('.py','python'),
-            ('.qml','qml'),
-            ('.rb','ruby'),
-            ('.scheme','scheme'),
-            ('.sh','sh'),
-            ('.tcl','tcl'),
-            ('texinfo','texinfo'),
-            ('.txt','None'),
-            ('.vb','vbnet'),
-            ('verilog','verilog'),
-            ('vhdl','vhdl'),
-            ('.xml','xml'),
-            )
 class FindAndReplaceDlg( QDialog):
     """ Find and replace dialog """
     find = pyqtSignal(unicode,bool,bool,bool,bool)
     replace = pyqtSignal(unicode,bool,bool,bool,bool)
     replaceAll = pyqtSignal(unicode,bool,bool,bool,bool)
-    
+
     def __init__(self, parent=None):
         super(FindAndReplaceDlg, self).__init__(parent)
 
@@ -182,70 +136,65 @@ class FindAndReplaceDlg( QDialog):
         self.replaceButton.setEnabled(enable)
         self.replaceAllButton.setEnabled(enable)
 
-class Window( QMainWindow):    
+class Window( QMainWindow):
     def __init__(self, parent):
-        global isMAEMO
         QMainWindow.__init__(self,None)
         self.parent = parent
 
         #Initialization of the plugin system
         init_plugin_system()
-            
+
         #Got the enabled plugin
         self.settings =  QSettings()
         self.enabled_plugins = []
         for plugin in find_plugins():
-            if bool(self.settings.value(plugin.__name__)):
-                self.enabled_plugins.append(plugin)      
+            if self.settings.contains(plugin.__name__):
+                if self.settings.value(plugin.__name__) == '2':
+                    print 'Enable plugin ', plugin
+                    self.enabled_plugins.append(plugin()) #FIXME
 
         self.findAndReplace = FindAndReplaceDlg()
         self.setupFileMenu()
         self.setupHelpMenu()
-        self.setupEditor()
-        
+
+
         try:
             self.setAttribute( Qt.WA_Maemo5AutoOrientation, True)
             self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
-            isMAEMO = True
-        except:
-            isMAEMO = False
 
-        #Resize window if not maemo
-        if not isMAEMO:
+            #Speed Hack for Maemo Kinetic scrolling
+            self.area =  QScrollArea(self)
+            try:
+                scroller = self.area.property("kineticScroller") #.toPyObject()
+                scroller.setEnabled(True)
+            except:
+                scroller = None
+            self.setupEditor(self.area)
+            self.area.setWidget(self.editor)
+            self.area.setWidgetResizable(True)
+            self.setCentralWidget(self.area)
+
+        except AttributeError, err:
+            print 'Not on maemo', err
+            #Resize window if not maemo
             self.resize(800, 600)
-            
-        self.area =  QScrollArea(self)
-        try:
-            scroller = self.area.property("kineticScroller") #.toPyObject()
-            scroller.setEnabled(True)
-        except:
-            scroller = None
-            
-        #speed hack
-        self.editor.scroller = scroller
-        self.editor.area = self.area
-        self.area.setWidget(self.editor)
-        self.area.setWidgetResizable(True)
-#        self.area.takeWidget()
-        
-#        self.setCentralWidget(self.editor)
-        self.setCentralWidget(self.area)
-
+            self.setupEditor()
+            self.setCentralWidget(self.editor)
 
     def fileSave(self):
         try:
             self.editor.save()
         except (IOError, OSError), e:
              QMessageBox.warning(self, "KhtEditor -- Save Error",
-                    "Failed to save %s: %s" % (self.filename, e))
+                    "Failed to save %s: %s" % (self.editor.filename, e))
 
 
     def saveAsFile(self):
         filename =  QFileDialog.getSaveFileName(self,
                         "KhtEditor -- Save File As",
-                        self.editor.filename, u'Python file(*.py);;'  
-                                            + u'Text file(*.txt);;' 
-                                            + u'C File(*.c);;' 
+                        self.editor.filename, u'Python file(*.py);;'
+                                            + u'Text file(*.txt);;'
+                                            + u'C File(*.c);;'
                                             + u'C++ File(*.cpp)')
         if not (filename == ''):
             self.editor.filename = filename
@@ -269,65 +218,29 @@ class Window( QMainWindow):
 #             QTimer.singleShot(100, self.editor.load)
             self.setWindowTitle( QFileInfo(self.editor.filename).fileName())
 #             QTimer.singleShot(100, self.loadHighlighter)
-            self.loadHighlighter(filename)
+#            self.loadHighlighter(filename)
+
         except (IOError, OSError), e:
              QMessageBox.warning(self, "KhtEditor -- Load Error",
                     "Failed to load %s: %s" % (filename, e))
 
-    def setupEditor(self):
-#        font =  QFont()
-#        font.setFamily("Courier")
-#        font.setFixedPitch(True)
-#        font.setPointSize(12)
-
+    def setupEditor(self,scroller=None):
         self.editor = editor.KhtTextEdit(self)
-#        self.editor.setFont(font)
-        #self.editor_frame = editor_frame.Frame(self.editor)
+        self.editor.scroller = scroller
+        self.editor.show_progress.connect(self.show_progress)
         self.setupToolBar()
-#        self.language = self.detectLanguage()
-#        self.loadHilighter()
         self.editor.document().modificationChanged.connect(self.do_documentChanged)
 
-    def loadHighlighter(self,filename=None):
-        global isMAEMO
-        filename = self.editor.filename
-        language = self.detectLanguage(filename)
-        #Return None if language not yet implemented natively in KhtEditor
-        if language == 'python':
-            from syntax.python_highlighter import Highlighter
-            if isMAEMO:
-                self.setAttribute( Qt.WA_Maemo5ShowProgressIndicator,True)
-            QApplication.processEvents()
-            self.highlighter = Highlighter(self.editor.document())
-            QApplication.processEvents()
-            if isMAEMO:
-                self.setAttribute( Qt.WA_Maemo5ShowProgressIndicator,False)
-        elif (language != None) and (language != 'None'):
-            from syntax.generic_highlighter import Highlighter
-            if isMAEMO:
-                self.setAttribute( Qt.WA_Maemo5ShowProgressIndicator,True)
-            QApplication.processEvents()
-            self.highlighter = Highlighter(self.editor.document(),language)
-            QApplication.processEvents()
-            if isMAEMO:
-                self.setAttribute( Qt.WA_Maemo5ShowProgressIndicator,False)            
-        else:
-            from syntax import pygments_highlighter
-            if isMAEMO:
-                self.setAttribute( Qt.WA_Maemo5ShowProgressIndicator,True)
-            QApplication.processEvents()
-            self.highlighter = pygments_highlighter.Highlighter(self.editor.document(),unicode(filename))
-            QApplication.processEvents()
-            if isMAEMO:
-                self.setAttribute( Qt.WA_Maemo5ShowProgressIndicator,False)            
 
-    
-    def detectLanguage(self,filename):
-        for extension,lang in LANGUAGES:
-            if filename.endswith(extension.lower()):
-                 return lang
-        return None
-                    
+    @pyqtSlot(bool)
+    def show_progress(self,show):
+        try:
+            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, show)
+        except AttributeError:
+            print 'Not on maemo'
+
+
+
     def setupToolBar(self):
         self.toolbar = self.addToolBar('Toolbar')
 
@@ -381,11 +294,19 @@ class Window( QMainWindow):
         self.tb_findagain.triggered.connect(self.findAndReplace.findClicked)
         self.addAction(self.tb_findagain)
 
-        #Hook for plugins to add buttons :
+        #Plugins menu
+        self.tb_plugin_menu = QMenu(self)
+        self.tb_plugin_button = QToolButton(self)
+        self.tb_plugin_button.setText('Plugins')
+        self.tb_plugin_button.setMenu(self.tb_plugin_menu)
+        self.tb_plugin_button.setPopupMode(QToolButton.InstantPopup)
+        self.tb_plugin_button.setCheckable(False)
+        self.toolbar.addWidget(self.tb_plugin_button)
+
+        #Hook for plugins to add buttons in combo box:
         for plugin in filter_plugins_by_capability('toolbarHook',self.enabled_plugins):
             print 'Found 1 Plugin for toolbarHook'
-            plg = plugin()
-            plg.do_toolbarHook(self)
+            plugin.do_toolbarHook(self.tb_plugin_menu)
 
     def setupFileMenu(self):
         fileMenu =  QMenu(self.tr("&File"), self)
@@ -404,19 +325,23 @@ class Window( QMainWindow):
 
         helpMenu.addAction(self.tr("&About"), self.do_about)
 
+    @pyqtSlot()
     def do_about(self):
         self.parent.about(self)
 
+    @pyqtSlot(long)
     def do_gotoLine(self, line):
         print 'goto line:'+str(line)
         self.editor.gotoLine(line)
 
+    @pyqtSlot()
     def do_find(self):
         self.findAndReplace.find.connect(self.editor.find)
         self.findAndReplace.replace.connect(self.editor.replace)
         self.findAndReplace.replaceAll.connect(self.editor.replace_all)
         self.findAndReplace.show()
 
+    @pyqtSlot()
     def do_execute(self):
         print "execute"
         #ask for save if unsaved
@@ -427,16 +352,16 @@ class Window( QMainWindow):
               fileHandle = open('/tmp/khteditor.tmp', 'wb')
               fileHandle.write('#!/bin/sh\n')
               fileHandle.write('cd '+os.path.dirname(unicode(self.editor.filename).encode('utf-8'))+' \n')
-              language = self.detectLanguage(self.editor.filename)
+              language = self.editor.detectLanguage(self.editor.filename)
               #Crappy way to handle that
               if language == 'python':
                 fileHandle.write("python \'"+unicode(self.editor.filename).encode('utf-8') + "\'\n")
               elif language == 'qml':
-                fileHandle.write("qmlviewer \'"+unicode(self.editor.filename).encode('utf-8') + "\' -fullscreen\n")      
+                fileHandle.write("qmlviewer \'"+unicode(self.editor.filename).encode('utf-8') + "\' -fullscreen\n")
               elif language in ('c', 'cpp'):
                 fileHandle.write("make %s\nchmod +x %s\n./%s" % (unicode(self.editor.filename).encode('utf-8'),\
                                                                  unicode(self.editor.filename).encode('utf-8'),\
-                                                                 unicode(self.editor.filename).encode('utf-8')))       
+                                                                 unicode(self.editor.filename).encode('utf-8')))
               elif language != None:
                 fileHandle.write(language+" \'"+unicode(self.editor.filename).encode('utf-8') + "\' \n")
               else:
@@ -450,6 +375,7 @@ class Window( QMainWindow):
           commands.getoutput("chmod 777 /tmp/khteditor.tmp")
           Popen('/usr/bin/osso-xterm /tmp/khteditor.tmp',shell=True,stdout=None)
 
+    @pyqtSlot()
     def lineCountUpdate(self):
         cursor = self.editor.textCursor()
         self.lineCount.setText("L.%d C.%d" % (cursor.blockNumber()+1,
@@ -458,12 +384,14 @@ class Window( QMainWindow):
     def closeEvent(self,event):
         self.editor.closeEvent(event)
 
+    @pyqtSlot()
     def do_fullscreen(self):
         if self.isFullScreen():
             self.showMaximized()
         else:
             self.showFullScreen()
 
+    @pyqtSlot(bool)
     def do_documentChanged(self,changed):
         if changed == True:
             self.setWindowTitle('*'+ QFileInfo(self.editor.filename).fileName())
